/*****************************************************************************************
 *  Copyright (C), 2011
 *  SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  NAME:    ORVA_GET_DIST_PARAM_NAMES
 *  PURPOSE: Construct a lookup dataset to obtain the display name 
 *           for all fitting parameters. It will alos translate the 
 *           DisplayName's to the language and locale of the server. 
 *
 *  SUPPORT: jariou
 *
 *  PARAMETERS:
 *    xmlFile = The full name of the distributions.xml file       
 *    xmlMsgs = The base dataset tu use for internationalization of XML files 
 *    outData = The name of the lookup dataset that it creates 
 *              (work.sparmnames or work.fparmnames)
 *    type    = SEV or FREQ
 *
 *****************************************************************************************/
 %macro orva_get_dist_param_names(xmlFile =, xmlMsgs =, outData =, type = SEV);
   %local tmp_data;       /* to hold the list of datasets to be deleted at the end */
   %let tmp_data = %str();

   /* Create a transformation map file */
   data  _null_;
      rc = filename("map",,"temp");
   run;
   
   data _null_;
      file map;
      put '<SXLEMAP version="1.0">';

      %if &type eq SEV %then %do;         /* ......Table.Name........|............Column.Names..............*/
         %_orva_map_mixed_dists();       /* MIXED_DISTRIBUTIONS |BODY | TAIL | isKriFitable |*/
         %_orva_map_sev_dist_params();   /* DIST_PARAMS         |Distribution | DisplayName | DatasetName |*/
         %_orva_map_sev_dists_only();    /* DISTS_ONLY          |DISTRIBUTION | isKriFitable | Name |Description |*/
         %_orva_map_tail_dists_only();   /* TAIL_DISTS_ONLY     |DISTRIBUTION | Name | Description |*/
         %_orva_map_tail_params();       /* TAIL_DIST_TPARAMS   |PREFIX | DISPLAYNAME | ORDER |*/
         %_orva_map_tail_dist_params();  /* TAIL_DIST_TPARAMS   |PREFIX | DISPLAYNAME | DATASETNAME |*/
      %end;
      %else %do;
         %_orva_map_freq_dist_params();  /* DISTRIBUTIONS    | DISTRIBUTION | DISPLAYNAME | DATASETNAME | */
         %_orva_map_freq_dists_only();   /* FREQ_DISTS_ONLY  | DISTRIBUTION | */
      %end;

      put '</SXLEMAP>';
   run;

   %_orva_get_&type._xml_info(outParams = &outData, outDists = &outData._2);

   proc sort data = &outData; 
      by 
         Distribution 
         DatasetName
         ; 
   run;
   
   %orvu_delete_data(data = &tmp_data);
%mend orva_get_dist_param_names;

%macro _orva_map_mixed_dists();
      put '   <TABLE name="mixed_distributions">';
      put '      <TABLE_XPATH > /Extensions/Distributions/Severity </TABLE_XPATH>';
      put '      <COLUMN name="Body" replace="YES">';
      put '         <XPATH> /Extensions/Distributions/Severity@id</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>12</LENGTH>';
      put '      </COLUMN>';
      put '      <COLUMN name="Tail" replace="YES">';
      put '         <XPATH> /Extensions/Distributions/Severity/MixedBodyTail@id</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>12</LENGTH>';
      put '      </COLUMN>';
      put '      <COLUMN name="isKriFitable" retain="NO">';
      put '         <XPATH> /Extensions/Distributions/Severity@isKriFitable</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>32</LENGTH>';
      put '      </COLUMN>';
      put '      <COLUMN name="Mixed" retain="NO">';
      put '         <XPATH> /Extensions/Distributions/Severity/MixedBodyTail@id</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>32</LENGTH>';
      put '      </COLUMN>';
      put '   </TABLE>';
%mend  _orva_map_mixed_dists;

%macro _orva_map_sev_dist_params();
      put '   <TABLE name="sev_dist_params">';
      put '      <TABLE_XPATH > /Extensions/Distributions/Severity/Param </TABLE_XPATH>';
      put '      <COLUMN name="Distribution" retain="YES">';
      put '         <XPATH> /Extensions/Distributions/Severity@id</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>32</LENGTH>';
      put '      </COLUMN>';
      put '      <COLUMN name="DisplayName" retain="NO">';
      put '         <XPATH> /Extensions/Distributions/Severity/Param@name</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>64</LENGTH>';
      put '      </COLUMN>';
      put '      <COLUMN name="DatasetName" retain="NO">';
      put '         <XPATH> /Extensions/Distributions/Severity/Param@datasetName</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>32</LENGTH>';
      put '      </COLUMN>';
      put '   </TABLE>';
%mend _orva_map_sev_dist_params;

%macro _orva_map_sev_dists_only();
      put '   <TABLE name="sev_dists_only">';
      put '      <TABLE_XPATH > /Extensions/Distributions/Severity </TABLE_XPATH>';
      put '      <COLUMN name="Distribution" retain="YES">';
      put '         <XPATH> /Extensions/Distributions/Severity@id</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>32</LENGTH>';
      put '      </COLUMN>';
      put '      <COLUMN name="isKriFitable" retain="NO">';
      put '         <XPATH> /Extensions/Distributions/Severity@isKriFitable</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>32</LENGTH>';
      put '      </COLUMN>';
      put '      <COLUMN name="Name" retain="NO">';
      put '         <XPATH> /Extensions/Distributions/Severity@name</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>64</LENGTH>';
      put '      </COLUMN>';
      put '      <COLUMN name="Description" retain="NO">';
      put '         <XPATH> /Extensions/Distributions/Severity@description</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>64</LENGTH>';
      put '      </COLUMN>';
      put '   </TABLE>';
%mend _orva_map_sev_dists_only;

%macro _orva_map_tail_dists_only();  
      put '   <TABLE name="tail_dists_only">';
      put '      <TABLE_XPATH > /Extensions/Distributions/SeverityTail </TABLE_XPATH>';
      put '      <COLUMN name="Distribution" retain="YES">';
      put '         <XPATH> /Extensions/Distributions/SeverityTail@id</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>32</LENGTH>';
      put '      </COLUMN>';
      put '      <COLUMN name="Name" retain="NO">';
      put '         <XPATH> /Extensions/Distributions/SeverityTail@name</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>64</LENGTH>';
      put '      </COLUMN>';
      put '      <COLUMN name="Description" retain="NO">';
      put '         <XPATH> /Extensions/Distributions/SeverityTail@description</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>64</LENGTH>';
      put '      </COLUMN>';
      put '   </TABLE>';
%mend _orva_map_tail_dists_only;  

%macro _orva_map_tail_params();
      put '   <TABLE name="tail_dist_TParams">';
      put '      <TABLE_XPATH > /Extensions/Distributions/SeverityTail/TParam </TABLE_XPATH>';
      put '      <COLUMN name="Prefix" retain="YES">';
      put '         <XPATH> /Extensions/Distributions/SeverityTail@id</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>32</LENGTH>';
      put '      </COLUMN>';
      put '      <COLUMN name="DisplayName" retain="YES">';
      put '         <XPATH> /Extensions/Distributions/SeverityTail/TParam@name</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>64</LENGTH>';
      put '      </COLUMN>';
      put '      <COLUMN name="order" retain="YES">';
      put '         <XPATH> /Extensions/Distributions/SeverityTail/TParam@order</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>32</LENGTH>';
      put '      </COLUMN>';
      put '   </TABLE>';
%mend _orva_map_tail_params;

%macro _orva_map_tail_dist_params();
      put '   <TABLE name="tail_dist_Params">';
      put '      <TABLE_XPATH > /Extensions/Distributions/SeverityTail/Param </TABLE_XPATH>';
      put '      <COLUMN name="Prefix" retain="YES">';
      put '         <XPATH> /Extensions/Distributions/SeverityTail@id</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>32</LENGTH>';
      put '      </COLUMN>';
      put '      <COLUMN name="DisplayName" retain="YES">';
      put '         <XPATH> /Extensions/Distributions/SeverityTail/Param@name</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>64</LENGTH>';
      put '      </COLUMN>';
      put '      <COLUMN name="DatasetName" retain="YES">';
      put '         <XPATH> /Extensions/Distributions/SeverityTail/Param@datasetName</XPATH>';
      put '         <TYPE>character</TYPE>';
      put '         <DATATYPE>string</DATATYPE>';
      put '         <LENGTH>32</LENGTH>';
      put '      </COLUMN>';
      put '   </TABLE>';
%mend _orva_map_tail_dist_params;

%macro _orva_map_freq_dist_params();         
         put '   <TABLE name="freq_dist_params">';
         put '      <TABLE_XPATH > /Extensions/Distributions/Frequency/Param </TABLE_XPATH>';
         put '      <COLUMN name="Distribution" retain="YES">';
         put '         <XPATH> /Extensions/Distributions/Frequency@id</XPATH>';
         put '         <TYPE>character</TYPE>';
         put '         <DATATYPE>string</DATATYPE>';
         put '         <LENGTH>32</LENGTH>';
         put '      </COLUMN>';
         put '      <COLUMN name="DisplayName" retain="NO">';
         put '         <XPATH> /Extensions/Distributions/Frequency/Param@name</XPATH>';
         put '         <TYPE>character</TYPE>';
         put '         <DATATYPE>string</DATATYPE>';
         put '         <LENGTH>64</LENGTH>';
         put '      </COLUMN>';
         put '      <COLUMN name="DatasetName" retain="NO">';
         put '         <XPATH> /Extensions/Distributions/Frequency/Param@datasetName</XPATH>';
         put '         <TYPE>character</TYPE>';
         put '         <DATATYPE>string</DATATYPE>';
         put '         <LENGTH>32</LENGTH>';
         put '      </COLUMN>';
         put '   </TABLE>';
%mend _orva_map_freq_dist_params;         

%macro _orva_map_freq_dists_only();
         put '   <TABLE name="freq_dists_only">';
         put '      <TABLE_XPATH > /Extensions/Distributions/Frequency </TABLE_XPATH>';
         put '      <COLUMN name="Distribution" retain="YES">';
         put '         <XPATH> /Extensions/Distributions/Frequency@id</XPATH>';
         put '         <TYPE>character</TYPE>';
         put '         <DATATYPE>string</DATATYPE>';
         put '         <LENGTH>32</LENGTH>';
         put '      </COLUMN>';
         put '      <COLUMN name="isCertain" retain="NO">';
         put '         <XPATH> /Extensions/Distributions/Frequency@isCertain</XPATH>';
         put '         <TYPE>character</TYPE>';
         put '         <DATATYPE>string</DATATYPE>';
         put '         <LENGTH>32</LENGTH>';
         put '      </COLUMN>';
         put '      <COLUMN name="name" retain="NO">';
         put '         <XPATH> /Extensions/Distributions/Frequency@name</XPATH>';
         put '         <TYPE>character</TYPE>';
         put '         <DATATYPE>string</DATATYPE>';
         put '         <LENGTH>32</LENGTH>';
         put '      </COLUMN>';
         put '      <COLUMN name="description" retain="NO">';
         put '         <XPATH> /Extensions/Distributions/Frequency@description</XPATH>';
         put '         <TYPE>character</TYPE>';
         put '         <DATATYPE>string</DATATYPE>';
         put '         <LENGTH>32</LENGTH>';
         put '      </COLUMN>';
         put '   </TABLE>';
%mend _orva_map_freq_dists_only;

%macro _orva_get_freq_xml_info(outParams = , outDists = );
   %_orva_get_standard_parameters(distType = FREQ);

   data &outParams; 
      set _StandardParams; 
   run;

   data &outDists; 
      set xml_lib.freq_dists_only;

      if missing(isCertain) then do;
         isCertain = "false";
      end;

      tmpName = Name;
      TranslatedName = sasmsg("sashelp.orvxmsg", Name, "noquote");

      if upcase(tmpName) = upcase(TranslatedName) then do;
         TranslatedName = tmpName;
      end;

      tmpDescription = Description;
      TranslatedDescription = sasmsg("sashelp.orvxmsg", Description, "noquote");

      if upcase(tmpDescription) = upcase(TranslatedDescription) then do;
         TranslatedDescription = tmpDescription;
      end;

      rename 
               TranslatedName        = Name 
               TranslatedDescription = Description
               ;
      drop Name tmpName Description tmpDescription;
   run;
%mend _orva_get_freq_xml_info;

%macro _orva_get_standard_parameters(distType = );
   /* Load the XML libname engine with the xml file and the transformation map */
   filename myxml "&xmlFile";

   libname  
            xml_lib      XML 
            xmlfileref = myxml 
            xmlmap     = map
            ; 

   /* Get the list of "standard" distribution params */
   data _StandardParams;
      length TranslatedDisplayName $32 tmpDisplayName $32;
      set xml_lib.&distType._dist_params;

      Distribution = reverse(scan(reverse(Distribution), 1));

      tmpDisplayName = DisplayName;
      TranslatedDisplayName = sasmsg("&xmlMsgs", tmpDisplayName, "noquote");


      if upcase(TranslatedDisplayName) = upcase(DisplayName) then do; 
         TranslatedDisplayName = DisplayName;
      end;

      rename 
            translatedDisplayName = DisplayName 
            DisplayName           = DisplayNameKey
            ;

      drop  tmpDisplayName;
   run;
   %let tmp_data = &tmp_data. _StandardParams;
%mend _orva_get_standard_parameters;

%macro _orva_get_sev_xml_info(outParams = , outDists = );
   %_orva_get_standard_parameters(distType = SEV);
   /* Get a list Mixed Distributions */
   data _aMixedSevs;
      length Distribution $32;
      set xml_lib.mixed_distributions;

     if not missing(Mixed) then do;
         Distribution = strip(tail) || strip(body);
     
         if missing(isKriFitable) then do;
           isKriFitable = "true";
         end;     

        output;
     end;
     
      drop Mixed; 
   run;
   %let tmp_data = &tmp_data. _aMixedSevs;

   /* Get the tail parameters */
   data _bTailTParams;
      length TranslatedDisplayName $32 tmpDisplayName $32;
      set xml_lib.tail_dist_tparams;
      tmpDisplayName = DisplayName;
      TranslatedDisplayName = sasmsg("&xmlMsgs", tmpDisplayName, "noquote");

      if upcase(TranslatedDisplayName) = upcase(DisplayName) then do; 
         TranslatedDisplayName = DisplayName;
      end;

      rename translatedDisplayName = DisplayName  DisplayName = DisplayNameKey ;
      drop  tmpDisplayName;
   run;
   %let tmp_data = &tmp_data. _bTailTParams;

   data _cTailParams;
      length TranslatedDisplayName $32 tmpDisplayName $32;
      set xml_lib.tail_dist_params;
      tmpDisplayName = DisplayName;
      TranslatedDisplayName =sasmsg("&xmlMsgs", tmpDisplayName, "noquote");

      if upcase(TranslatedDisplayName) = upcase(DisplayName) then do; 
         TranslatedDisplayName = DisplayName;
      end;

      rename translatedDisplayName = DisplayName DisplayName = DisplayNameKey;
      drop tmpDisplayName;
   run;
   %let tmp_data = &tmp_data. _cTailParams;

   /* merge tail tparams with mixed distributions */
   proc sql noprint; 
      CREATE TABLE 
                  _dmerge_tail AS  
      SELECT 
                  * 
      FROM
                  _aMixedSevs m 
      LEFT JOIN 
                  _bTailTParams p 
      ON
                  m.tail eq p.prefix
                  ;
   quit;
   proc sort 
            data = _dmerge_tail
            ; 
      by Distribution Order; 
   run;
   %let tmp_data = &tmp_data. _dmerge_tail;


   /* get param count form body distributions */
   proc sort 
            data = _StandardParams 
            out  = _eparams
            ; 
      by Distribution; 
   run;
   %let tmp_data = &tmp_data. _eparams;

   data _fparamCount; 
      set _eparams; 
      by Distribution;
      retain parmcount 0;

      Body = Distribution;

      if first.Distribution then do;
         parmcount = 1;
      end;
      else do;
         parmcount = parmcount + 1;
      end;

      if last.Distribution then do;
      output;
      end;

      keep Body parmcount; 
   run;
   %let tmp_data = &tmp_data. _fparamCount;

   proc sql noprint; 
      CREATE TABLE 
                  _gmerge_tail2 
      AS SELECT 
                  * 
      FROM
                  _dmerge_tail m 
      LEFT JOIN 
                  _fparamCount 
      ON
                  m.Body = _fparamCount.Body
                  ;
   quit;
   %let tmp_data = &tmp_data. _gmerge_tail2;

   data _hmerge_tail3; 
      set _gmerge_tail2;
      length DatasetName $32;
      DatasetName = "PARAM_" || strip(put(Parmcount+order,best.));
      keep Distribution DisplayName DisplayNameKey DatasetName; 
   run;
   %let tmp_data = &tmp_data. _hmerge_tail3;

   /* merge tail params with mixed distributions and concat with above */
   proc sql noprint; 
      CREATE TABLE 
                  _imerge_tail_x 
      AS SELECT 
                  * 
      FROM
                  _aMixedSevs m 
      LEFT JOIN 
                  _cTailParams p 
      ON
                  m.tail eq p.prefix
                  ;
   quit;
   %let tmp_data = &tmp_data. _imerge_tail_x;

   data _jmerge_tail4; 
      set 
            _hmerge_tail3 
            _imerge_tail_x(keep= Distribution DisplayName DisplayNameKey DatasetName)
            ; 
   run;
   %let tmp_data = &tmp_data. _jmerge_tail4;

   /* merge Mixed Dists with Standard Params */
   proc sql noprint; 
      CREATE TABLE 
                  _kmerge_body 
      AS SELECT
                  * 
      FROM
                  _aMixedSevs m 
      LEFT JOIN
                  _StandardParams p 
      ON
                  m.Body = p.Distribution
                  ;
   quit;
   %let tmp_data = &tmp_data. _kmerge_body;

   data _lmerge_body2; 
      set _kmerge_body (drop=Body Tail); 
   run;
   %let tmp_data = &tmp_data. _lmerge_body2;

   /* put all together */
   data &outParams; 
      set 
            _StandardParams 
            _lmerge_body2 
            _jmerge_tail4
            ; 
   run;

   data _sev_dists_only;
      set  xml_lib.sev_dists_only ;
   run;
   %let tmp_data = &tmp_data. _sev_dists_only;

   data _Tail_dists_only;
      set xml_lib.tail_dists_only;       

      Tail = Distribution;
      TailDescription = sasmsg("&orvxds", Description, "noquote");

      if upcase(TailDescription) = upcase(Description) then do;
         TailDescription = Description;
      end;

      TailName = sasmsg("&orvxds", Name, "noquote");

      if upcase(TailName) = upcase(Name) then do;
         TailName = Name;
      end;

      keep Tail TailName TailDescription ;
   run;
   %let tmp_data = &tmp_data. _Tail_dists_only;

   data _all_tail_info;
      merge _aMixedSevs _Tail_dists_only;
      by Tail;
   run;
   %let tmp_data = &tmp_data. _all_tail_info;

   proc sort
            data = _aMixedSevs
            ;
      by Body;
   run;

   proc sort
            data = _all_tail_info
            ;
      by Body;
   run;

   proc sort 
            data = _sev_dists_only
            ;
      by Distribution;
   run;
   
   proc sql noprint; 
      CREATE TABLE 
                  _all_tail_plus_body_keys 
      AS  
      SELECT 
                  *
      FROM
                  _all_tail_info m 
      LEFT JOIN 
                  _sev_dists_only p 
      ON
                  m.body eq p.distribution
                  ;
   quit;
   %let tmp_data = &tmp_data. _all_tail_plus_body_keys;

   data _all_tail_translated;
      set  _all_tail_plus_body_keys(keep=Distribution isKriFitable TailName TailDescription Name Description );
      tmpKey = Name;
      Name = sasmsg("&orvxds", name, "noquote");

      if upcase(Name) = upcase(tmpKey) then do;
         Name = catx(" ",tmpKey,TailName);
      end;
      else do;
         Name = catx(" ",Name,TailName);
      end;

      tmpKey = Description;
      Description = sasmsg("&orvxds", Description, "noquote");

      if upcase(Description) = upcase(tmpKey) then do;
         Description = catx(" ", tmpKey, TailDescription);
      end;
      else do;
         Description = catx(" ",Description, TailDescription);
      end;

      keep Distribution isKriFitable Description Name;
   run;
   %let tmp_data = &tmp_data. _all_tail_translated;

   data _all_body_translated; 
      set  
         xml_lib.sev_dists_only; 

      if missing(isKriFitable) then do;
         isKriFitable = "true";
      end;

      tmpKey = Name;
      Name = sasmsg("&orvxds", name, "noquote");

      if upcase(Name) = upcase(tmpKey) then do;
         Name = tmpKey;
      end;

      tmpKey = Description;
      Description = sasmsg("&orvxds", Description, "noquote");

      if upcase(Description) = upcase(tmpKey) then do;
         Description = tmpKey;
      end;

      keep 
            Distribution 
            isKriFitable 
            Description 
            Name
            ;
   run;
   %let tmp_data = &tmp_data. _all_body_translated;

   data &outDists; 
      set  
            _all_body_translated  
            _all_tail_translated
            ;
   run;
%mend _orva_get_sev_xml_info;
