/*****************************************************************************************
 * $Id: orvl_int_model_scenario.sas,v 1.1.4.40.2.13.2.1.8.1 2012/05/10 17:40:56 fengye Exp $
 *  Copyright (C), 2006
 *  SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *  NAME:    orvl_int_model_scenario                                             
 *  PURPOSE: Performs loading of the model scenario datasets.
 *  SUPPORT: mimuha
 *
 *  PARAMETERS:
 *    INLIB           = Libref for the input datasets
 *    OUTLIB          = Libref for the output datasets
 *    DATA_VERSION_ID = Version uid
 *    SEQ_LIB         = The libref for the sequences dataset
 *****************************************************************************************/
%macro orvl_int_model_scenario(INLIB, OUTLIB, DATA_VERSION_ID, SEQ_LIB);

   data _null_;
      call symput('TYP_NM', sasmsg("&orvlds", "ims_type_scen_effect_m", "noquote"));
   run;
   data _null_;
      call symput('HEADER_SC', sasmsg("&orvlds", "ims_ref_missing_scenario_title", "noquote", "&TYP_NM"));
      call symput('HEADER_SE', sasmsg("&orvlds", "ims_ref_missing_scenario_event_title", "noquote", "&TYP_NM"));
      call symput('HEADER_BL', sasmsg("&orvlds", "ims_ref_missing_int_bl_title", "noquote", "&TYP_NM"));
      call symput('HEADER_RC', sasmsg("&orvlds", "ims_ref_missing_int_rc_title", "noquote", "&TYP_NM"));
      call symput('HEADER_BLS', sasmsg("&orvlds", "ims_ref_missing_std_bl_title", "noquote", "&TYP_NM"));
      call symput('HEADER_RCS', sasmsg("&orvlds", "ims_ref_missing_std_rc_title", "noquote", "&TYP_NM"));
      call symput('HEADER_PR', sasmsg("&orvlds", "ims_ref_missing_process_title", "noquote", "&TYP_NM"));
      call symput('HEADER_LE', sasmsg("&orvlds", "ims_ref_missing_legal_entity_title", "noquote", "&TYP_NM"));
      call symput('HEADER_C1', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "1"));
      call symput('HEADER_C2', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "2"));
      call symput('HEADER_C3', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "3"));
      call symput('HEADER_C4', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "4"));
      call symput('HEADER_C5', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "5"));
      call symput('HEADER_C6', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "6"));
      call symput('HEADER_C7', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "7"));
      call symput('HEADER_C8', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "8"));
      call symput('HEADER_C9', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "9"));
      call symput('HEADER_C10', sasmsg("&orvlds", "ims_ref_missing_class_x_title", "noquote", "&TYP_NM", "10"));
   run;
   
   /* 
    * *************************************************
    * Make sure the required dataset has records. 
    * If it doesn't, orvl_ds_exists sets LOAD_STATUS to 2.
    * *************************************************
    */
   %orvl_ds_exists(&INLIB..scenario_effects, "&TYP_NM", false);
   %let input_count = &ds_nobs;
   %if &input_count eq 0 %then %do;
      %orvl_cmn_status_success;
      %return;
   %end;

   /*
   %orvl_log_output_wrap("STATUS - %sysfunc(sasmsg(&orvlds, ims_testing_model_scens_status, noquote))");
   
   %orvl_int_model_scenario_pcheck(ScenarioDataIn=&INLIB..model_scenario, ScenarioDataOut=WORK.model_scenario_ck);
   */

   %orvl_ds_exists(&OUTLIB..SCENARIO_EFFECT, "&TYP_NM", false);
   %let existing_main_count = &ds_nobs;
   %if &existing_main_count eq 0 %then %do;
      %let if_modify_1 = if 0 then modify;
      %let if_modify_2 = if 0 then modify;
      %let existing_alt_count = 0;
   %end;
   %else %do;
      %let if_modify_1 = modify key=FEED_REF_ID;

      %orvl_ds_exists(&OUTLIB..MODEL_SCENARIO_SEV_PARMS, "&TYP_NM", false);
      %let existing_alt_count = &ds_nobs;
      %if &existing_alt_count eq 0 %then %do;
         %let del_index_2 = ;
         %let if_modify_2 = if 0 then modify;
      %end;
      %else %do;
         %let if_modify_2 = modify key=SEV_IDX;
      %end;
   %end;
   data _null_;
      format item $400.;
      item = "proc sql noprint; drop index FEED_REF_ID on &OUTLIB..SCENARIO_EFFECT; quit;";
      call symput('del_index_1', item);
      item = "proc sql noprint; drop index SEV_IDX on &OUTLIB..MODEL_SCENARIO_SEV_PARMS; quit;";
      call symput('del_index_2', item);
   run;
   proc sql noprint; create unique index FEED_REF_ID on &OUTLIB..SCENARIO_EFFECT (FEED_REF_ID); quit;
   proc sql noprint; create unique index SEV_IDX on &OUTLIB..MODEL_SCENARIO_SEV_PARMS (SCENARIO_EFFECT_UID, PARM_NUM); quit;

   %orvl_log_output_wrap("STATUS PROGRESS2 TOTAL=&input_count FILE='&TYP_NM'");

   /*
    * ******************
    * Process the items. 
    * ******************
    */
   /* Get the starting uids for items to insert. */
   %orvdb_seq_incrementer(&SEQ_LIB, LOAD_Scenario_Details_SEQ, dbload, INC=&input_count);
   proc sql noprint;
      select VALUE Into :DETAILS_ID From WORK.VAR_SEQUENCES Where NAME = 'dbload';
   quit;
   %orvdb_seq_incrementer(&SEQ_LIB, Folder_SEQ, dbload, INC=&input_count);
   proc sql noprint;
      select VALUE Into :EFFECT_ID From WORK.VAR_SEQUENCES Where NAME = 'dbload';
   quit;

   proc summary data=&OUTLIB..MODEL_SCENARIO_SEV_PARMS MAX;
      ID SEV_PARM_UID;
      OUTPUT OUT=WORK.TMP_ID_DS;
   run;
   %orvl_ds_exists(WORK.TMP_ID_DS, "ID_FOR_SEV_PARMS", false);
   %if &ds_nobs ne 0 %then %do;
      proc sql noprint;
         select SEV_PARM_UID Into :PARM_ID From WORK.TMP_ID_DS;
      quit;
   %end;
   %else %do;
      data _null_;
         call symput('PARM_ID', '0');
      run;
   %end;

   %orvl_cmn_status_ds_prep(WORK.LOAD_STATUS_DETAILS);

   data WORK.tmp_eff (KEEP= SCENARIO_EFFECT_UID EFFECT_NAME SCENARIO_EVENT_UID);
      set &OUTLIB..SCENARIO_EFFECT;
   run;
   
   /* Create a temporary dataset with the new ids. */
   data WORK.TMP_SCENARIO_DETAILS_ME (KEEP= SCENARIO_DETAILS_UID
                                            FEED_REF_ID 
                                            PARENT_UID
                                            TYPE_CD
                                            INTERNAL_BL_UID
                                            INTERNAL_RC_UID
                                            BASEL_BL_UID
                                            BASEL_RC_UID
                                            PROCESS_UID
                                            LEGAL_ENTITY_UID
                                            CLASS_1
                                            CLASS_2
                                            CLASS_3
                                            CLASS_4
                                            CLASS_5
                                            CLASS_6
                                            CLASS_7
                                            CLASS_8
                                            CLASS_9
                                            CLASS_10
                                            PROBABILITY
                                            DATETIME_LOADED
                                            VERSION_UID)
        &OUTLIB..SCENARIO_EFFECT
        &OUTLIB..MODEL_SCENARIO_SEV_PARMS
        WORK.LOAD_STATUS_DETAILS
        %orvl_log_ds_init();
      length ORVAR_REASON_OBS_PREFIX $40. VALID_DISTS $1024. S_DIST $30. in_desc $256.;
      format SCENARIO_TYPE $1. BL_AND_RC_TYPE $1. FREQUENCY_DIST $32.;
      format ADD_UPDATE_DELETE $1.;
      set &INLIB..scenario_effects (rename=(Scenario_Effect_ID=in_id
                                            Scenario_Effect_Name=in_name
                                            Scenario_Effect_Description=in_desc
                                            Scenario_Effect_Probability=in_prob
                                            Scenario_Event_ID=in_event_ref_id
                                            Internal_BL_ID=in_bl_ref_id
                                            Internal_RC_ID=in_rc_ref_id
                                            /* Process_ID=in_process_ref_id */
                                            /* Legal_Entity_ID=in_legal_entity_ref_id */
                                            ));

      FEED_REF_ID = in_id;

      FPARM0 = FPARM0; /* Initialize */
      FPARM1 = FPARM1; /* Initialize */
      FPARM2 = FPARM2;  /* Initialize */
      in_freq_dist = FREQUENCY_DIST;
      in_freq_parm0 = FPARM0;
      in_freq_parm1 = FPARM1;
      in_freq_parm2 = FPARM2;

      retain details_id_val effect_id_val sev_parm_id_val;
      
      if _N_ = 1 then do;
         details_id_val = symgetn('DETAILS_ID');
         if (details_id_val eq .) then do;
            details_id_val = 1;
         end;
         effect_id_val = symgetn('EFFECT_ID');
         if (effect_id_val eq .) then do;
            effect_id_val = 1;
         end;
         sev_parm_id_val = symgetn('PARM_ID') + 1;
         if (sev_parm_id_val eq .) then do;
            sev_parm_id_val = 1;
         end;
         declare hash scenarioHash(dataset: "&OUTLIB..SCENARIO", hashexp: 10);
         rc = scenarioHash.defineKey('SCENARIO_UID');
         rc = scenarioHash.defineData('SCENARIO_TYPE', 'BL_AND_RC_TYPE', 'EVENT_TYPE');
         rc = scenarioHash.defineDone();
         declare hash eventHash(dataset: "&OUTLIB..SCENARIO_EVENT", hashexp: 10);
         rc = eventHash.defineKey('FEED_REF_ID');
         rc = eventHash.defineData('SCENARIO_EVENT_UID', 'SCENARIO_UID', 'FREQUENCY_DIST', 'FREQUENCY_PARM0', 'FREQUENCY_PARM1', 'FREQUENCY_PARM2');
         rc = eventHash.defineDone();
         declare hash effectHash(dataset: "WORK.tmp_eff", hashexp: 8);
         rc = effectHash.defineKey('EFFECT_NAME', 'SCENARIO_EVENT_UID');
         rc = effectHash.defineData('SCENARIO_EFFECT_UID');
         rc = effectHash.defineDone();
         declare hash blHash(dataset: "&OUTLIB..INTERNAL_BUSINESSLINE", hashexp: 10);
         rc = blHash.defineKey('FEED_REF_ID');
         rc = blHash.defineData('INTERNAL_BL_UID');
         rc = blHash.defineDone();
         declare hash rcHash(dataset: "&OUTLIB..INTERNAL_RISK_CATEGORY", hashexp: 10);
         rc = rcHash.defineKey('FEED_REF_ID');
         rc = rcHash.defineData('INTERNAL_RC_UID');
         rc = rcHash.defineDone();
         declare hash blBaselHash(dataset: "&OUTLIB..BASEL_BUSINESSLINE_I", hashexp: 10);
         rc = blBaselHash.defineKey('FEED_REF_ID');
         rc = blBaselHash.defineData('BASEL_BL_UID');
         rc = blBaselHash.defineDone();
         declare hash rcBaselHash(dataset: "&OUTLIB..BASEL_RISK_CATEGORY_I", hashexp: 10);
         rc = rcBaselHash.defineKey('FEED_REF_ID');
         rc = rcBaselHash.defineData('BASEL_ER_UID');
         rc = rcBaselHash.defineDone();
         declare hash processHash(dataset: "&OUTLIB..PROCESS", hashexp: 10);
         rc = processHash.defineKey('FEED_REF_ID');
         rc = processHash.defineData('PROCESS_UID');
         rc = processHash.defineDone();
         declare hash legalEntityHash(dataset: "&OUTLIB..LEGAL_ENTITY", hashexp: 10);
         rc = legalEntityHash.defineKey('FEED_REF_ID');
         rc = legalEntityHash.defineData('LEGAL_ENTITY_UID');
         rc = legalEntityHash.defineDone();
         declare hash distSev(dataset: "WORK._SDISTS", hashexp: 6);
         rc = distSev.defineKey('S_DIST');
         rc = distSev.defineData('S_DIST', 'S_NUM');
         rc = distSev.defineDone();
         %do i = 1 %to 10;
             declare hash class&i.Hash(dataset: "&OUTLIB..CLASS_&i.", hashexp: 10);
             rc = class&i.Hash.defineKey('FEED_REF_ID');
             rc = class&i.Hash.defineData("CLASS_&i._UID");
             rc = class&i.Hash.defineDone();
             call missing(CLASS_&i._UID);
         %end;
         declare hiter iter_s('distSev');
         %orvl_cmn_obs_exists_hash(OBS_EXISTS_EXISTING_COUNT=&existing_main_count, 
                                   HASH_KEY='FEED_REF_ID',
                                   HASH_DATA='effect_id_val');
         call missing(FEED_REF_ID, SCENARIO_EVENT_UID, SCENARIO_UID, FREQUENCY_DIST);
         call missing(FREQUENCY_PARM0, FREQUENCY_PARM1, FREQUENCY_PARM2, SCENARIO_TYPE, BL_AND_RC_TYPE, EVENT_TYPE);
         call missing(INTERNAL_BL_UID, INTERNAL_RC_UID, BASEL_BL_UID);
         call missing(BASEL_ER_UID, PROCESS_UID, LEGAL_ENTITY_UID, S_DIST, S_NUM, SCENARIO_EFFECT_UID);
      end;
      else do;
         details_id_val = details_id_val + 1;
         effect_id_val = effect_id_val + 1;
         sev_parm_id_val = sev_parm_id_val + 1;
      end;

      %orvl_log_ds_modify();
      &if_modify_1 &OUTLIB..SCENARIO_EFFECT;
      main_iorc = _IORC_;
      %orvl_cmn_status_ds_modify(WORK.LOAD_STATUS_DETAILS);

      put "STATUS PROGRESS2 COUNT_INC";
      %orvl_cmn_user_cancelled_check();
      status_cancel = symgetn('ORVAR_CANCEL');
      if status_cancel ne 0 then stop;

      if ((ADD_UPDATE_DELETE eq '') or (&existing_main_count eq 0)) then ADD_UPDATE_DELETE = 'A';
      ADD_UPDATE_DELETE = upcase(ADD_UPDATE_DELETE);

      /* Create a flag that will be used to determine if every item was found. */
      doOutputSevParams = 0;
      doOutput = 1;
      onErrorFail = 1;
            
      if (ADD_UPDATE_DELETE ne 'D') then do;
         FEED_REF_ID = in_event_ref_id;
         rc = eventHash.find();
         if (rc ne 0) then do;
            %orvl_find_reference(rc, "&HEADER_SE", WORK.LOAD_STATUS_DETAILS);
            doOutput = 0;
         end;
         else do;
            rc = scenarioHash.find();
            if (rc ne 0) then do;
               %orvl_find_reference(rc, "&HEADER_SC", WORK.LOAD_STATUS_DETAILS);
               doOutput = 0;
            end;
            else if (strip(SCENARIO_TYPE) eq "B") then do;
               /*%orvl_log_output('Bucket-based effect... Skipping.');*/
               doOutput = 0;
            end;
         end;

         if doOutput eq 1 then do;
            /* Just make sure the variables get created if they are not in the input datasets. */
            %do i = 1 %to 10;   
               CLASS_&i. = CLASS_&i.;
            %end;

            /* Assign whatever variables are available. */
            EFFECT_NAME = in_name;
            EFFECT_DESC = '';
            if (in_desc ne '') then EFFECT_DESC = in_desc;
            if (EVENT_TYPE eq 2) then do;
               EFFECT_PROB = -1;
               PROBABILITY = -1;
            end;
            else do;
               EFFECT_PROB = 1;
               PROBABILITY = 1;
               if (in_prob ne .) then do;
                  EFFECT_PROB = in_prob;
                  PROBABILITY = in_prob;
               end;
            end;
            SEVERITY_DIST = SEVERITY_DIST;
            SEVERITY_STATUS_REASON = '';
            SIMPLE_FREQUENCY_DIST = in_freq_dist;
            SIMPLE_FREQUENCY_PARM0 = in_freq_parm0;
            SIMPLE_FREQUENCY_PARM1 = in_freq_parm1;
            SIMPLE_FREQUENCY_PARM2 = in_freq_parm2;
            if (EFFECT_ORDER eq .) then EFFECT_ORDER = 0;
            TYPE_CD = 'E';
            VERSION_UID = &DATA_VERSION_ID;
            
            /* Find the Business Line and Risk Category. */
            if (strip(BL_AND_RC_TYPE) eq "I") then do;
               FEED_REF_ID = in_bl_ref_id;
               rc = blHash.find();
               if (FEED_REF_ID ne .) then do;
                  %orvl_find_reference(rc, "&HEADER_BL", WORK.LOAD_STATUS_DETAILS);
               end;
         
               FEED_REF_ID = in_rc_ref_id;
               rc = rcHash.find();
               if (FEED_REF_ID ne .) then do;
                  %orvl_find_reference(rc, "&HEADER_RC", WORK.LOAD_STATUS_DETAILS);
               end;
            end;
            else do;
               FEED_REF_ID = in_bl_ref_id;
               rc = blBaselHash.find();
               if (FEED_REF_ID ne .) then do;
                  %orvl_find_reference(rc, "&HEADER_BLS", WORK.LOAD_STATUS_DETAILS);
               end;
         
               FEED_REF_ID = in_rc_ref_id;
               rc = rcBaselHash.find();
               if (FEED_REF_ID ne .) then do;
                  %orvl_find_reference(rc, "&HEADER_RCS", WORK.LOAD_STATUS_DETAILS);
                  BASEL_RC_UID = BASEL_ER_UID;
               end;
            end;
      
            /* Find the Process. */
            Process_ID = Process_ID;
            FEED_REF_ID = Process_Id;
            rc = processHash.find();
            if (FEED_REF_ID ne .) then do;
               %orvl_find_reference(rc, "&HEADER_PR", WORK.LOAD_STATUS_DETAILS);
            end;
            
            /* Find the Legal Entity. */
            FEED_REF_ID = Legal_Entity_Id;
            rc = legalEntityHash.find();
            if (FEED_REF_ID ne .) then do;
               %orvl_find_reference(rc, "&HEADER_LE", WORK.LOAD_STATUS_DETAILS);
            end;
     
            %do i = 1 %to 10;   
               if (CLASS_&i. ne .) then do;
                  /* Find the CLASS_# Refs. */
                  FEED_REF_ID = CLASS_&i.;
                  rc = class&i.Hash.find();
                  %orvl_find_reference(rc, "&&HEADER_C&i", WORK.LOAD_STATUS_DETAILS);
                  CLASS_&i. = CLASS_&i._UID;
               end;
            %end;

            FEED_REF_ID = in_id;
      
            /* Are the parameters valid? */
            /*
            if (in_is_valid ne 1) then do;
               %orvl_cmn_record_error(40, _N_, 
                  in_reason, 
                  "&TYP_NM", WORK.LOAD_STATUS_DETAILS);
            end;
            */
      
            %orvl_cmn_test_required_value(FEED_REF_ID, %str(Scenario_Effect_ID), _N_, "&TYP_NM", WORK.LOAD_STATUS_DETAILS);
            %orvl_cmn_test_required_string(EFFECT_NAME, %str(Scenario_Effect_Name), _N_, "&TYP_NM", WORK.LOAD_STATUS_DETAILS);
            %orvl_cmn_test_required_value(SCENARIO_EVENT_UID, %str(Scenario_Event_ID), _N_, "&TYP_NM", WORK.LOAD_STATUS_DETAILS);
            %orvl_cmn_test_required_string(SEVERITY_DIST, %str(Severity_Dist), _N_, "&TYP_NM", WORK.LOAD_STATUS_DETAILS);

            %orvl_cmn_obs_exists_test_msg(OBS_EXISTS_EXISTING_COUNT=&existing_main_count, SRC_NAME="&TYP_NM", DO_MSG_ONLY=false);

            /* Make sure the Severity Distributions are valid. */
            if (strip(SEVERITY_DIST) ne "") then do;
               S_DIST = SEVERITY_DIST;
               rc = distSev.find();
               if (rc ^= 0) then do;
                  VALID_DISTS = " ";
                  iter_rc = iter_s.first();
                  do while (iter_rc = 0);
                     VALID_DISTS = strip(VALID_DISTS) || " " || S_DIST;
                     iter_rc = iter_s.next();
                  end;
                  VALID_DISTS = sasmsg("&orvlds", "ims_invalid_dist_for_sev", "noquote", 
                                       strip(SEVERITY_DIST), strip(VALID_DISTS));
                  %orvl_cmn_record_error(43, _N_, strip(VALID_DISTS), "&TYP_NM", WORK.LOAD_STATUS_DETAILS);
               end;
            end;         
         end;
      end;
   
      /* Output the item and the parameters only if all of the referenced components 
       * were found, and the parameters are valid. */
      if doOutput eq 1 then do;
         if (ADD_UPDATE_DELETE eq 'A') then do;
            DATETIME_LOADED = &ORVAR_DTTM_LOADED;
            rc = effectHash.find();
            if (rc ne 0) then do;
               if ((&existing_main_count eq 0) or (main_iorc eq %sysrc(_DSENMR)) or (main_iorc eq %sysrc(_DSENOM))) then do;
                  SCENARIO_EFFECT_UID = effect_id_val;
                  FEED_REF_ID = in_id;
                  output &OUTLIB..SCENARIO_EFFECT;
                  _ERROR_ = 0;
                  effectHash.add();
                  doOutputSevParams = 1;
               end;
               else if ((main_iorc eq %sysrc(_SOK)) or (main_iorc eq %sysrc(_SENOCHN))) then do;
                  %orvl_cmn_obs_exists_test_msg(DO_MSG_ONLY=true, SRC_NAME="&TYP_NM");
               end;
            end;
            _ERROR_ = 0;
            SCENARIO_DETAILS_UID = details_id_val;
            PARENT_UID = SCENARIO_EFFECT_UID;
            FEED_REF_ID = in_id;
            output WORK.TMP_SCENARIO_DETAILS_ME;
            _ERROR_ = 0;
         end;
         else if (ADD_UPDATE_DELETE eq 'D') then do;
            /*
            if (main_iorc eq %sysrc(_SOK)) then remove &OUTLIB..SCENARIO_EFFECT;
            _ERROR_ = 0;
            */
         end;
         else do;
            DATETIME_LOADED = &ORVAR_DTTM_LOADED;
            /*
            if (main_iorc eq %sysrc(_SOK)) then replace &OUTLIB..SCENARIO_EFFECT;
            else %orvl_cmn_test_match_found(_N_, "&TYP_NM", main_iorc, WORK.LOAD_STATUS_DETAILS);
            */
         end;
   
         if doOutputSevParams eq 1 then do;
            Array sev_parm_ref[100] Sparm1 - Sparm100;
            i = 1;
            do while (i <= 100);
               PARM_NUM = i;
               PARM_VALUE = .;
               &if_modify_2 &OUTLIB..MODEL_SCENARIO_SEV_PARMS;
               sev_parms_iorc = _IORC_;
               if (ADD_UPDATE_DELETE eq 'A') then do;
                  DATETIME_LOADED = &ORVAR_DTTM_LOADED;
                  if (sev_parm_ref[i] ne .) then do;
                     PARM_VALUE = sev_parm_ref[i];
                     if ((&existing_alt_count eq 0) or (sev_parms_iorc eq %sysrc(_DSENMR)) or (sev_parms_iorc eq %sysrc(_DSENOM))) then do;
                        SEV_PARM_UID = sev_parm_id_val;
                        output &OUTLIB..MODEL_SCENARIO_SEV_PARMS;
                        sev_parm_id_val = sev_parm_id_val + 1;
                     end;
                  end;
                  _ERROR_ = 0;
               end;
               else if (ADD_UPDATE_DELETE eq 'D') then do;
                  /* Handled above with the delete? */
               end;
               else do;
                  if (sev_parm_ref[i] ne .) then do;
                     DATETIME_LOADED = &ORVAR_DTTM_LOADED;
                     PARM_VALUE = sev_parm_ref[i];
                     if (sev_parms_iorc eq %sysrc(_SOK)) then replace &OUTLIB..MODEL_SCENARIO_SEV_PARMS;
                     else %orvl_cmn_test_match_found(_N_, "&TYP_NM", sev_parms_iorc, WORK.LOAD_STATUS_DETAILS);
                  end;
               end;
               i = i + 1;
               if ((i <= 100) and (sev_parm_ref[i] eq .) and (PARM_VALUE eq .)) then i = 102;
            end;
         end;
      end;
      else _ERROR_ = 0;
   run;

   %orvl_ds_exists(&OUTLIB..SCENARIO, "SCENARIO_COUNT", false);
   %orvl_int_scenario_col_def(WORK.TMP_SCENARIO_DETAILS_ME, 
                              &OUTLIB..SCENARIO_EFFECT,
                              &OUTLIB..SCENARIO_EVENT,
                              &OUTLIB..SCENARIO,
                              WORK.TMP_SCENARIO_DETAILS_DEF, 
                              &ds_nobs,
                              &SEQ_LIB);

   %orvl_cmn_user_cancelled_ck_wrap();
   %if &ORVAR_CANCEL ne 0 %then %goto DONE;

   /* Append the temp datasets to the original datasets. */
   %if (&LOAD_STATUS eq 1) %then %do;
      %orvl_modify_scenario_details("&TYP_NM [Column Defs]", &OUTLIB, WORK.TMP_SCENARIO_DETAILS_DEF);
      %orvl_cmn_user_cancelled_ck_wrap();
      %if &ORVAR_CANCEL ne 0 %then %goto DONE;
   %end;

   %if (&LOAD_STATUS eq 1) %then %do;
      %orvl_modify_scenario_details("&TYP_NM [Details]", &OUTLIB, WORK.TMP_SCENARIO_DETAILS_ME);
      %orvl_cmn_user_cancelled_ck_wrap();
      %if &ORVAR_CANCEL ne 0 %then %goto DONE;
   %end;

%DONE:
   &del_index_1;
   &del_index_2;

   %orvl_log_output_wrap("STATUS PROGRESS2 DONE");

   /* Handle the final signaling. */
   %orvl_cmn_status_processing;

%mend orvl_int_model_scenario;
