/*-------------------------------------------------------------------
 * $Id: orvr_report_setup.sas,v 1.1.4.10.2.21.2.9.2.1.2.1 2012/04/30 14:39:03 jogwhi Exp $
 * Copyright (C), 2004
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 * NAME:    orvr_report_setup                                             
 * PURPOSE: Setup general stuff needed by all reports. 
 *
 *          title1 is reserved for the Company Title and us set
 *          by macro %cleartitles;
 *      
 * SUPPORT: stchri
 *
 *
 * Input:
 *    fileref - fileref for HTML output.
 *    model_uid - specified what project we're dealing with.
 *    data_root - The root path to the data store, ending with the file separator character
 *
 *
 * NOTES: Assumes caller has set the following librefs:
 *        varprj, vardata
 *        In general, caller of this macro should call orvr_report_cleanup at its conclusion.
 *-------------------------------------------------------------------*/
%macro orvr_report_setup(fileref       =,
                          model_uid    =,
                          outputtype   =,
                          style        =DEFAULT,
                          graphics     =false,
                          data_root    =);
 
    option noovp nodate;
     
    /*** Create fake data for empty reports ***/
    data _fake_;
        length fakevar $10; label fakevar = '00'X;
        fakevar= ' ';
    run;

    /*** fix for template issue where can't overwrite template in sasuser area ***/
    ods path(prepend) work.xxx(update);

    /*** select CURFMT and DEFFMT format ***/
    %readReportFormat();
    %global ORVR_DATASTORE_ROOT;
    %let ORVR_DATASTORE_ROOT = &data_root;
        
    /*** specifications for titles and stuff ***/
    %global titleSpec1;
        %let titleSpec1 = j=center color=CX996600 height=14pt ;
    %global titleSpec2;
        %let titleSpec2 = j=center color=black height=12pt ;
    %global titleSpec3;
        %let titleSpec3 = j=center color=gray height=10pt ;
    %global titleSpec4;
        %let titleSpec4 = j=center height=10pt;
    %global titleSpec5;
        %let titleSpec5 = j=center height=8pt;

        %macro cleartitles;
                title; title1; title2; title3; title4; title5; title6; title7; title8; 
                title1 &titleSpec1 "&COMPANYTITLE";
        %mend;
        
        
        
    %global footnoteSpec1;
        %let footnoteSpec1 = j=right height=8pt color=black "%sysfunc(sasmsg(&orvrds, rs_report_generated_time_footer, noquote)) : %sysfunc(putn(%sysfunc(datetime()), &DATETIMEFMT))";
    %global reportSpec;
    %let reportSpec = nowd SPLIT="*" style(column)=[font_size=2] headline headskip CENTER;
        
        /*** Start ODS Session ***/ 
    %if %substr(%upcase(&outputtype),1,3) = PDF %then %do;
           options orientation=landscape leftmargin=0.01in rightmargin=0.01in 
                                         topmargin=0.01in bottommargin=0.01in;
        title;
     %end;
     %else %do; 
        %cleartitles;
     %end;

        
        %if %substr(%upcase(&outputtype),1,3) = HTM %then %do;
           %if "&graphics" eq "true" %then %do;
              ods tagsets.htmlpanel gpath="&_VARLIB" body=&fileref style=&style codebase="&_VARLIB";
           %end;
           %else %do;
              ods &outputtype body=&fileref style=&style path="&_VARCONTENTS" gpath="&_VARCONTENTS" (url="./") codebase="&_VARLIB";
           %end;
        %end;
        %else %do;
           %if %substr(%upcase(&outputtype),1,3) = PDF %then %do;
              ods &outputtype body=&fileref style=&style pdftoc=1;
           %end;
           %else %do;
              ods &outputtype body=&fileref style=&style;
           %end;
        %end;
    ods listing close;

 
        /********************************************
          Create macro variables for the following 
            MODEL_NAME - currnt model name
            FULL_NAME  - of model's author
            TODAY_DATE - date report is run 
          *******************************************/
    %global MODEL_NAME VAR_USER_UID VERSION_UID;
        data _NULL_; set varprj.MODEL;
                where MODEL_UID = &model_uid;
            call symput("MODEL_NAME",strip(model_name));
                call symput("VAR_USER_UID",var_user_uid);
                call symput("VERSION_UID",version_uid);
        run;
        
        data var_user;
                set VARDATA.VAR_USER;
                where VAR_USER_UID=&VAR_USER_UID;
        run;
        
    %global TODAY_DATE THIRD_DIM NUMCCVS RS_FULL_NAME;
    
        data _NULL_;
          merge var_user varprj.MODEL;
          by VAR_USER_UID;
          
          
          call symput ("FIRST_NAME",strip(FIRST_NAME));
          call symput ("MIDDLE_NAME",strip(MIDDLE_NAME));
          call symput ("LAST_NAME",strip(LAST_NAME));
          
      
          format dt &DATEFMT;
          dt = date();
          call symput("TODAY_DATE",left(put(dt,&DATEFMT)));
        run;
        
        %let rs_full_name = %sysfunc(sasmsg(&orvrds, rs_full_name_order, noquote, 
                       %bquote(&first_name),%bquote(&middle_name), %bquote(&last_name)));  
        
       
   
    
        
    proc datasets lib=work memtype=data nolist;
       delete var_user;
    quit;
    
    %global MAPNAME0 MAPNAME1 MAPNAME2 MAPNAME2_ORIG CCV_NAMES2 NUM_CCV2;

         /* Determine the number of dimensions ------------*/
    %let NUMCCVS=0;
    proc contents noprint data=varprj.cc out=_cc; run;
    %let dsid=%sysfunc(open(_cc,I));
    %if &dsid ^= 0 %then %do;
      %let NUMCCVS=%sysfunc(attrn(&dsid,nlobs));
      %let rc=%sysfunc(close(&dsid));
    %end;
    %if &NUMCCVS eq 0 %then %return;
    %let NUMCCVS= %eval(&numccvs-1);

        /* generate a list of third dimension names ---------*/
    %let THIRD_DIM=0;
    
    %if &NUMCCVS > 2 %then %do;
      %let THIRD_DIM=1;
      data _NULL_; set varprj.ccmapval_2; where ccvuid_2 >= 0;
       
       length ccv $ 10000;
       retain ccv ;

       if ( _N_ = 1 ) then ccv = strip(ccvvalue_2);
       else ccv = strip(ccv) || "#" || strip(ccvvalue_2);

       call symput("ccv_names2", strip(ccv));
       call symput("num_ccv2", _n_);
      run;
    %end;
    %else %do;
      %let ccv_names2=;
      %let num_ccv2=0;
    %end;

        /*** create macros for MAPNAME0 and MAPNAME1 ***/

        data _NULL_;
          set varprj.ccmapval_0; where CCVUID_0 = -999;
          call symput("MAPNAME0",strip(CCVVALUE_0));
        run;
        data _NULL_;
          set varprj.ccmapval_1; where CCVUID_1 = -999;
          call symput("MAPNAME1",strip(CCVVALUE_1));
        run;
        
        %if &THIRD_DIM %then %do;
            data _NULL_;
              set varprj.ccmapval_2; where CCVUID_2 = -999;
              call symput("MAPNAME2",strip(CCVVALUE_2));
            run;
        %end;

      %let fixedstr=;
      %let MAPNAME2_ORIG=&MAPNAME2;
      %fix_string(&MAPNAME2);
      %let MAPNAME2=&fixedstr;
     

     /* Create macros for VARCOMP, VARORG, and VARMETA --*/
     /* first get the version suffix --------------------*/
     data _NULL_; set VARDATA.data_version;
     where version_uid = &version_uid;
     call symput("dir_suffix", compress(dir_suffix));
     run;
     %global VARCOMP VARORG VARMETA;
     %let VARCOMP=VC&dir_suffix;
     %let VARORG=VO&dir_suffix;
     %let VARMETA=VT&dir_suffix;

 /***create mod_auth global message variable***/
    %global project;
    %global author;
    %let project = %sysfunc(sasmsg(&orvrds, project_title, noquote, &model_name));
    %let author = %sysfunc(sasmsg(&orvrds, author_title, noquote, &rs_full_name));
    
%mend orvr_report_setup;

%macro fix_string(string);
  data _null_;
    length replace $ 256;
    length c $ 1;
    string= strip("&string");
    stringl= length(string);
    do i=1 to stringl;
      c = substr(string,i,1);
          if notname(c) eq 0 then
            replace = strip(replace) || c;
      else  
        replace = strip(replace) || '_';
    end;
    call symput("FIXEDSTR",replace);
  run;
%mend;

%macro empty_report(phrase);

    proc report &reportSpec data=_fake_;  
            column fakevar;
            define fakevar / display "&phrase";
    run;quit;

%mend empty_report;

/*
 * The macro that reads the report format settings from metadata server 
 */
%macro readReportFormat();
	%global CURFMT;
	%let CURFMT = NLMNY32.2;
	%global DEFFMT;
	%let DEFFMT = NLNUM32.3;
	%global DATEFMT;
	%let DATEFMT = NLDATE.;
	%global DATETIMEFMT;
	%let DATETIMEFMT = NLDATMAP.;
	%global COMPANYTITLE;
	%let COMPANYTITLE= Unknown Company;
        %global COMPANYICON COMPANYBACKGROUND;
	
		 
	options metaserver     = &metaserver 
	        metaport       = &metaport
	        metarepository = &metarepo
/*	        metauser       = &metauser  */
/*	        metapass       = &metapass; */
	;
	
	%let software_component= &SOFTWARECOMP;   
	data _null_;
		length 	uri_sc 
				uri_appconfig 
				uri_prop 
				name 
				propName 
				propValue $256;
		uri_sc = '';
	  	nobj_sc = metadata_getnobj("omsobj:SoftwareComponent?@Name='&software_component'",1,uri_sc);
		rc3 = 1;
	    propSetIndex = 1;
	    uri_appconfig  = '';
	    appconfig_found = 0;
		name = "";
		uri_prop = "";
	    do while (rc3 > 0);
		    /*return value of metadata_getnasn: */
	        /* n: the # of associated objs*/
		    /* -1: unable to connect to the metadata sever*/
		    /* -3: No obj matched the URI */
		    /* -4: n is out of range */
	        rc3 = metadata_getnasn(uri_sc, "PropertySets", propSetIndex, uri_appconfig);
	        if rc3 > 0 then do;
	            rc2 = metadata_getattr(uri_appconfig, "Name", name);
	            if name = "Application Configuration" then do;
	                appconfig_found = 1;
			        ind = 1;
					rc1=1;
			        do while (rc1 >0);
			  		    rc1 = metadata_getnasn(uri_appconfig, "SetProperties", ind, uri_prop);
			  		    propName = "";
						propValue = "";
			  		    if rc1 > 0 then do;
						    rc0 = metadata_getattr(uri_prop, "name", propName);
					        if propName = "ReportCurrencyFormat" then do;
					            rc0 = metadata_getattr(uri_prop, "DefaultValue", propValue);
								call symput("CURFMT", trim(left(propValue)));
					        end;
					        if propName = "ReportCompanyTitle" then do;
					            rc0 = metadata_getattr(uri_prop, "DefaultValue", propValue);
								call symput("COMPANYTITLE", trim(left(propValue)));
					        end;
					        if propName = "ReportNumericFormat" then do;
					            rc0 = metadata_getattr(uri_prop, "DefaultValue", propValue);
								call symput("DEFFMT", trim(left(propValue)));
					        end;
					        if propName = "ReportDateTimeFormat" then do;
					            rc0 = metadata_getattr(uri_prop, "DefaultValue", propValue);
								call symput("DATETIMEFMT", trim(left(propValue)));
					        end;
					        if propName = "ReportDateFormat" then do;
					            rc0 = metadata_getattr(uri_prop, "DefaultValue", propValue);
								call symput("DATEFMT", trim(left(propValue)));
					        end;
                                                if propName = "ReportCompanyIcon" then do;
					            rc0 = metadata_getattr(uri_prop, "DefaultValue", propValue);
								call symput("CompanyIcon", "'" || trim(left(propValue)) || "'");
					        end;
                                                if propName = "ReportCompanyBackground" then do;
					            rc0 = metadata_getattr(uri_prop, "DefaultValue", propValue);
						    call symput("CompanyBackground", "'" || trim(left(propValue)) || "'");
					        end;
			  		    end;
					    ind = ind + 1;
			        end;	
	            end;
	        end;
	        else if rc3 eq -4 and propSetIndex eq 1 then do;
	            leave;
	        end;
	        propSetIndex = propSetIndex + 1;
	    end;
	run;
%mend readReportFormat;
